/*
 * Functions handling ethernet and sd card
 */
 //
// Listen for incoming Web Client Requests
//
void listenForClient()
{
  myClient = server.available();     // listen for incoming clients
  if (myClient)
  {
    while (myClient.connected())
    {
      if (myClient.available()) // read until no more characters - Chrome does 2 requests!
      {
        char c = myClient.read();
        Serial.write(c);
      }
      else
      {
        startHTTP();                   // send http response header
        fileToClient();                //Send contents of SD Card Text file to the client
        myClient.println("</html>");   // end of HTML
        break;
      }
    }
    delay(1);               // give the web browser time to receive the data
    myClient.stop();        // close the HTTP connection:
    Serial.println("client disconnected");
  }
}

//
// Respond to the HTTP request and print start of HTML page with headline
//
void startHTTP()
{
  myClient.println("HTTP/1.1 200 OK");
  myClient.println("Content-Type: text/html");
  myClient.println("Connection: close");  // connection closed after completion of the response
  myClient.println("Refresh: 30");        // refresh the page automatically every 30 sec
  myClient.println();
  myClient.println("<!DOCTYPE HTML>");
  myClient.println("<html>");
  myClient.println("<h2 style=\" color: blue;\">Netlogger_r5 by J Errington MSc 24 March 2016</h2>");
  myClient.print("<h3 style=\" color: blue;\">");
  myClient.print("selected sample rate in seconds: ");
  myClient.print(sInt);
  myClient.println("</h3>");
}

//
// Send contents of SD Card file to HTTP Client
//
void fileToClient()
{
  myFile = SD.open("test1.txt");             // Open the SD Card file
  myClient.print("<table style=\"color: blue; text-align: center; width: 800\">"); 
  myClient.print("<tr><th width=\"40%\">time since start</th><th width=\"20%\">elapsed minutes</th><th width=\"20%\">V1 mV</th><th width=\"20%\">V2 mV</th></tr >");
  while (myFile.available())                // Read data from the file until end of file
  {
    myClient.print("<tr>");                 // start HTML table row
    endOfLine = false;
    while (endOfLine == false)              // repeat for each part of dataset until end of line
    {
      myClient.print("<td>");               // start table data cell
      getData(charArray);                   // extract a data item from current line of file
      myClient.print(charArray);            // write data value
      myClient.print("</td>");              // end of table data cell
    }
    myClient.print("</tr>");                // end of HTML table row
  }
  myFile.close();
  myClient.print("</table>");               // end HTML table
  myClient.println("<br/>");       
  myClient.println("</html>");
}

//
// Read data from a file and parse into data items as the line is read. SemiColon separator.
// The data item is returned in a character array. No need to know how many data items in
// each line or how long the data items are. Called from fileToClient
//
void getData(char *dataItem)
{
  for ( byte i = 0; ; i++)           // read characters from file
  {
    char c = myFile.read();
    if (c != ';')                      // char is not a semi colon separator, so must be data
    {
      dataItem[i]  = c;                // add char to character array 
    }
    else                               // must be a semi colon separator - end of data item
    { 
      dataItem[i] = '\0';              // terminate the dataItem
      c = myFile.peek();               // peek at the next character in the file
      if (c == 13)                     // is the next char a CR - end of line?
      {                                
        endOfLine = true;              // YES! Set the end of line flag
        c = myFile.read();             // Dummy read to skip past CR
        c = myFile.read();             // Dummy read to skip past LF
      }
      //
      // If we get here, we have reached the end of the data item, and posiibly the end of the
      // line. So we have finished reading for now. Break out of the Read Loop and return to
      // the caller, with the current data item and the end of line flag set true, if
      // appropriate.
      // The file pointer is pointing at the first character of the next data item, or possibly,
      // we have reached the end of the file
      //     
      break;
    }
  }
}
